@@ -73,6 +73,8 @@ def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statemen
     brokerGroup.add_argument('-k', '--mqttkeepalive', required=False, type=int, default=60, help='Time between keep-alive messages')
     brokerGroup.add_argument('-mu', '--mqtt-username', type=str, dest='mqttusername', help='Username for MQTT broker', required=False)
     brokerGroup.add_argument('-mp', '--mqtt-password', type=str, dest='mqttpassword', help='Password for MQTT broker', required=False)
+    brokerGroup.add_argument('-mv', '--mqtt-version', type=str, dest='mqttversion', help='MQTT protocol version used', required=False,
+                             choices=['3.1', '3.1.1', '5'], default='3.1.1')
     brokerGroup.add_argument('--transport', required=False, default='tcp', choices=["tcp", 'websockets'], help='EXPERIMENTAL support for websockets transport')
     brokerGroup.add_argument('-s', '--use-tls', action='store_true', help='EXPERIMENTAL')
     brokerGroup.add_argument('--insecure', action='store_true', help='EXPERIMENTAL')
@@ -153,6 +155,13 @@ def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statemen
             handler.addFilter(util.DuplicateFilter())
     LOG.info('WeConnect-mqtt %s (using WeConnect-python %s)', __version__, __weconnect_version__)
 
+    if args.mqttversion == '3.1':
+        mqttVersion = paho.mqtt.client.MQTTv31
+    elif args.mqttversion == '5':
+        mqttVersion = paho.mqtt.client.MQTTv5
+    else:
+        mqttVersion = paho.mqtt.client.MQTTv311
+
     usetls = args.use_tls
     if args.cacerts:
         usetls = True
@@ -261,7 +270,7 @@ def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statemen
         else:
             convertTimezone = tz.gettz(args.convertTimes)
 
-    mqttCLient = WeConnectMQTTClient(clientId=args.mqttclientid, transport=args.transport, interval=args.interval,
+    mqttCLient = WeConnectMQTTClient(clientId=args.mqttclientid, protocol=mqttVersion, transport=args.transport, interval=args.interval,
                                      prefix=args.prefix, ignore=args.ignore, updateCapabilities=(not args.noCapabilities),
                                      updatePictures=args.pictures, selective=args.selective, listNewTopics=args.listTopics,
                                      republishOnUpdate=args.republishOnUpdate, pictureFormat=args.pictureFormat, topicFilterRegex=topicFilterRegex,
@@ -346,10 +355,10 @@ def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statemen
 
 
 class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-instance-attributes
-    def __init__(self, clientId=None, transport='tcp', interval=300, prefix='weconnect/0', ignore=0,  # pylint: disable=too-many-arguments
-                 updateCapabilities=True, updatePictures=True, selective=None, listNewTopics=False, republishOnUpdate=False,
+    def __init__(self, clientId=None, protocol=paho.mqtt.client.MQTTv311, transport='tcp', interval=300,  # pylint: disable=too-many-arguments
+                 prefix='weconnect/0', ignore=0, updateCapabilities=True, updatePictures=True, selective=None, listNewTopics=False, republishOnUpdate=False,
                  pictureFormat=None, topicFilterRegex=None, convertTimezone=None, timeFormat=None, withRawJsonTopic=False, passive=False):
-        super().__init__(client_id=clientId, transport=transport)
+        super().__init__(client_id=clientId, transport=transport, protocol=protocol)
         self.weConnect = None
         self.prefix = prefix
         self.interval = interval
@@ -374,10 +383,16 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
         self.withRawJsonTopic = withRawJsonTopic
         self.passive = passive
 
-        self.on_connect = self.on_connect_callback
-        self.on_message = self.on_message_callback
-        self.on_disconnect = self.on_disconnect_callback
-        self.on_subscribe = self.on_subscribe_callback
+        if protocol == paho.mqtt.client.MQTTv5:
+            self.on_connect = self.on_connect_callback_v5
+            self.on_message = self.on_message_callback
+            self.on_disconnect = self.on_disconnect_callback_v5
+            self.on_subscribe = self.on_subscribe_callback_v5
+        else:
+            self.on_connect = self.on_connect_callback
+            self.on_message = self.on_message_callback
+            self.on_disconnect = self.on_disconnect_callback
+            self.on_subscribe = self.on_subscribe_callback
 
         self.will_set(topic=f'{self.prefix}/mqtt/weconnectConnected', qos=1, retain=True, payload=False)
 
@@ -437,13 +452,13 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
         self.setConnected(connected=True)
         self.setError(code=WeConnectErrors.SUCCESS)
 
-    def updateWeConnect(self):  # noqa: C901
+    def updateWeConnect(self, reraise=False):  # noqa: C901
         if self.passive:
             return
         LOG.info('Update data from WeConnect')
         self.hasChanges = False
         try:
-            self.weConnect.update(updateCapabilities=self.updateCapabilities, updatePictures=self.updatePictures, selective=self.selective)
+            self.weConnect.update(updateCapabilities=self.updateCapabilities, updatePictures=self.updatePictures, selective=self.selective, force=True)
             self.setConnected(connected=True)
             self.setError(code=WeConnectErrors.SUCCESS)
             topic = f'{self.prefix}/mqtt/weconnectUpdated'
@@ -458,26 +473,34 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
                          payload=convertedTimeString)
             if topic not in self.topics:
                 self.addTopic(topic)
-        except errors.RetrievalError:
+        except errors.RetrievalError as error:
             self.setConnected(connected=False)
             errorMessage = f'Retrieval error during update. Will try again after configured interval of {self.interval}s'
             self.setError(code=WeConnectErrors.RETRIEVAL_FAILED, message=errorMessage)
             LOG.info(errorMessage)
-        except errors.APICompatibilityError:
+            if reraise:
+                raise error
+        except errors.APICompatibilityError as error:
             self.setConnected(connected=False)
             errorMessage = f'API compatibility error during update. Will try again after configured interval of {self.interval}s'
             self.setError(code=WeConnectErrors.API_COMPATIBILITY, message=errorMessage)
             LOG.info(errorMessage)
-        except errors.TemporaryAuthentificationError:
+            if reraise:
+                raise error
+        except errors.TemporaryAuthentificationError as error:
             self.setConnected(connected=False)
             errorMessage = f'Temporary authentification error during update. Will try again after configured interval of {self.interval}s'
             self.setError(code=WeConnectErrors.AUTHENTIFICATION, message=errorMessage)
             LOG.info(errorMessage)
-        except socket.error:
+            if reraise:
+                raise error
+        except socket.error as error:
             self.setConnected(connected=False)
             errorMessage = f'Socket error during update. Will try again after configured interval of {self.interval}s'
             self.setError(code=WeConnectErrors.RETRIEVAL_FAILED, message=errorMessage)
             LOG.info(errorMessage)
+            if reraise:
+                raise error
         if self.withRawJsonTopic and self.hasChanges:
             topic = f'{self.prefix}/rawjson'
             if topic not in self.topics:
@@ -599,6 +622,76 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
             print('Could not connect: %d', rc, file=sys.stderr)
             sys.exit(1)
 
+    def on_connect_callback_v5(self, mqttc, obj, flags, reasonCode, properties):  # noqa: C901
+        del mqttc  # unused
+        del obj  # unused
+        del flags  # unused
+        del properties
+
+        if reasonCode == 0:
+            LOG.info('Connected to MQTT broker')
+            if not self.passive:
+                topic = f'{self.prefix}/mqtt/weconnectForceUpdate_writetopic'
+                self.subscribe(topic, qos=2)
+                if topic not in self.topics:
+                    self.addTopic(topic, writeable=True)
+
+                topic = f'{self.prefix}/mqtt/weconnectUpdateInterval_s'
+                self.publish(topic=topic, qos=1, retain=True,
+                             payload=self.interval)
+                self.subscribe(topic + '_writetopic', qos=1)
+                if topic not in self.topics:
+                    self.addTopic(topic + '_writetopic', writeable=True)
+
+            self.setConnected()
+
+            self.updateWeConnect()
+        elif reasonCode == 128:
+            LOG.error('Could not connect (%d): Unspecified error', reasonCode)
+        elif reasonCode == 129:
+            LOG.error('Could not connect (%d): Malformed packet', reasonCode)
+        elif reasonCode == 130:
+            LOG.error('Could not connect (%d): Protocol error', reasonCode)
+        elif reasonCode == 131:
+            LOG.error('Could not connect (%d): Implementation specific error', reasonCode)
+        elif reasonCode == 132:
+            LOG.error('Could not connect (%d): Unsupported protocol version', reasonCode)
+        elif reasonCode == 133:
+            LOG.error('Could not connect (%d): Client identifier not valid', reasonCode)
+        elif reasonCode == 134:
+            LOG.error('Could not connect (%d): Bad user name or password', reasonCode)
+        elif reasonCode == 135:
+            LOG.error('Could not connect (%d): Not authorized', reasonCode)
+        elif reasonCode == 136:
+            LOG.error('Could not connect (%d): Server unavailable', reasonCode)
+        elif reasonCode == 137:
+            LOG.error('Could not connect (%d): Server busy. Retrying in 10 seconds', reasonCode)
+            time.sleep(10)
+            self.reconnect()
+        elif reasonCode == 138:
+            LOG.error('Could not connect (%d): Banned', reasonCode)
+        elif reasonCode == 140:
+            LOG.error('Could not connect (%d): Bad authentication method', reasonCode)
+        elif reasonCode == 144:
+            LOG.error('Could not connect (%d): Topic name invalid', reasonCode)
+        elif reasonCode == 149:
+            LOG.error('Could not connect (%d): Packet too large', reasonCode)
+        elif reasonCode == 151:
+            LOG.error('Could not connect (%d): Quota exceeded', reasonCode)
+        elif reasonCode == 154:
+            LOG.error('Could not connect (%d): Retain not supported', reasonCode)
+        elif reasonCode == 155:
+            LOG.error('Could not connect (%d): QoS not supported', reasonCode)
+        elif reasonCode == 156:
+            LOG.error('Could not connect (%d): Use another server', reasonCode)
+        elif reasonCode == 157:
+            LOG.error('Could not connect (%d): Server move', reasonCode)
+        elif reasonCode == 159:
+            LOG.error('Could not connect (%d): Connection rate exceeded', reasonCode)
+        else:
+            print('Could not connect: %d', reasonCode, file=sys.stderr)
+            sys.exit(1)
+
     def on_disconnect_callback(self, client, userdata, rc):
         del client
         del userdata
@@ -620,6 +713,59 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
                 finally:
                     time.sleep(10)
 
+    def on_disconnect_callback_v5(self, client, userdata, reasonCode, properties):  # noqa: C901
+        del client
+        del properties
+
+        if reasonCode == 0:
+            LOG.info('Client successfully disconnected')
+        elif reasonCode == 4:
+            LOG.info('Client successfully disconnected: %s', userdata)
+        elif reasonCode == 137:
+            LOG.error('Client disconnected: Server busy')
+            while True:
+                try:
+                    self.reconnect()
+                    break
+                except ConnectionRefusedError as e:
+                    LOG.error('Could not reconnect to MQTT-Server: %s, will retry in 10 seconds', e)
+                except socket.timeout:
+                    LOG.error('Could not reconnect to MQTT-Server due to timeout, will retry in 10 seconds')
+                except OSError as e:
+                    LOG.error('Could not reconnect to MQTT-Server: %s, will retry in 10 seconds', e)
+                finally:
+                    time.sleep(10)
+        elif reasonCode == 139:
+            LOG.error('Client disconnected: Server shutting down')
+            while True:
+                try:
+                    self.reconnect()
+                    break
+                except ConnectionRefusedError as e:
+                    LOG.error('Could not reconnect to MQTT-Server: %s, will retry in 10 seconds', e)
+                except socket.timeout:
+                    LOG.error('Could not reconnect to MQTT-Server due to timeout, will retry in 10 seconds')
+                except OSError as e:
+                    LOG.error('Could not reconnect to MQTT-Server: %s, will retry in 10 seconds', e)
+                finally:
+                    time.sleep(10)
+        elif reasonCode == 160:
+            LOG.error('Client disconnected: Maximum connect time')
+        else:
+            LOG.error('Client unexpectedly disconnected (%d), trying to reconnect', reasonCode)
+            while True:
+                try:
+                    self.reconnect()
+                    break
+                except ConnectionRefusedError as e:
+                    LOG.error('Could not reconnect to MQTT-Server: %s, will retry in 10 seconds', e)
+                except socket.timeout:
+                    LOG.error('Could not reconnect to MQTT-Server due to timeout, will retry in 10 seconds')
+                except OSError as e:
+                    LOG.error('Could not reconnect to MQTT-Server: %s, will retry in 10 seconds', e)
+                finally:
+                    time.sleep(10)
+
     def on_subscribe_callback(self, mqttc, obj, mid, granted_qos):
         del mqttc  # unused
         del obj  # unused
@@ -628,6 +774,17 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
         self.lastSubscribe = datetime.now()
         LOG.debug('sucessfully subscribed to topic')
 
+    def on_subscribe_callback_v5(self, mqttc, obj, mid, reasonCodes, properties):
+        del mqttc  # unused
+        del obj  # unused
+        del mid  # unused
+        del properties  # unused
+        if any(x in [0, 1, 2] for x in reasonCodes):
+            self.lastSubscribe = datetime.now()
+            LOG.debug('sucessfully subscribed to topic')
+        else:
+            LOG.error('Subscribe was not successfull (%s)', ', '.join(reasonCodes))
+
     def on_message_callback(self, mqttc, obj, msg):  # noqa: C901
         del mqttc  # unused
         del obj  # unused
